/*
 * Copyright (c) 2007 Koji Suzuki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "usbconfig.h"
#include "cdc162.h"

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))


union {
	uint16_t word;
	struct {
		uint8_t lo;
		uint8_t hi;
	};
} rate,sample_num;

static inline void set_rate() {
	TCCR0B = 0;
	TCCR1B = 0;
	TCNT0 = 0;
	TCNT1 = 0L;

	TCCR0A = (2<<WGM00);
	TCCR0B = (rate.hi & 0x7); // 1: 1/1 , 2:1/8, 3:1/64, 4:1/256, 5:1/1024
	OCR0A = rate.lo;
	TCCR1B = 1; // free running @ 1/1
}


#define nop()	__asm__ volatile ("nop" )

#define getc	usbcdc_getc
#define putc	usbcdc_putc
#define poll	usbcdc_poll

int main(void)
{
uint8_t ch;

    /* set system_clock pre scaler 1/8 => 1/1 */
    CLKPR = (1<<CLKPCE);
    CLKPR = 0;

    usbcdc_init();

    sei();
    for(;;) {
	    poll();
	    ch = getc();
	    if (ch == 'S') {
		    putc('U');
		    putc('S');
		    putc('B');
		    putc('1');
		    putc('6');
		    putc('2');
		    putc('B');
	    } else if ((ch == 'V') || (ch == 'v')) {
		    putc('1');
		    putc('0');
	    } else if ((ch == 0x1b) || (ch == '\n')) { // ESC,\n: nop
	    } else if (ch == 'r') { // set rate
		    rate.lo = getc();
		    rate.hi = getc();
		    putc('\r'); // ACK
#ifdef EEPROM_DEBUG
		    eeprom_write_byte((void *)0x7c, rate.lo);
		    eeprom_write_byte((void *)0x7d, rate.hi);
#endif
	    } else if (ch == 's') { // set samples
		    sample_num.lo = getc();
		    sample_num.hi = getc();
		    putc('\r'); // ACK
#ifdef EEPROM_DEBUG
		    eeprom_write_byte((void *)0x7e, sample_num.lo);
		    eeprom_write_byte((void *)0x7f, sample_num.hi);
#endif
	    } else if (ch == 'g') { // go 
		    set_rate();
		    while (sample_num.word) {
			    if (bit_is_set(TIFR0, OCF0A)) {
			    	bit_set(TIFR0, OCF0A);
				putc(TCNT1L);
				putc(TCNT1H);
				putc(TCNT1L);
				putc(TCNT1H);
		    		sample_num.word--;
			    }
			    //poll();
		    }
	    } else if (ch == 'h') { // go + 1
		    set_rate();
		    while (sample_num.word) {
			    if (bit_is_set(TIFR0, OCF0A)) {
			    	bit_set(TIFR0, OCF0A);
				getc();
				getc();
				getc();
				getc();
		    		sample_num.word--;
			    }
			    //poll();
		    }
	    } else if (ch == 'i') { // go + 2
		    set_rate();
		    while (sample_num.word) {
			    if (bit_is_set(TIFR0, OCF0A)) {
			    	bit_set(TIFR0, OCF0A);
				getc();
				getc();
				getc();
				getc();
				putc(TCNT1L);
				putc(TCNT1H);
				putc(TCNT1L);
				putc(TCNT1H);
		    		sample_num.word--;
			    }
			    //poll();
		    }
	    } else {
		    putc('?');
	    }
    }
    return 0;
}
